"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parse_1 = require("./parse");
const kebabRegex = /[A-Z\u00C0-\u00D6\u00D8-\u00DE]/g;
function kebabCase(pattern) {
    return pattern.replace(kebabRegex, match => '-' + match.toLowerCase());
}
function createStyle(selector, props, instance) {
    if (props === null) {
        return null;
    }
    const propertyNames = Object.keys(props);
    if (propertyNames.length === 0) {
        if (instance.config.preserveEmptyBlock)
            return selector + ' {}';
        return null;
    }
    const statements = [
        selector + ' {'
    ];
    propertyNames.forEach(propertyName => {
        const property = props[propertyName];
        const unwrappedProperty = String(typeof property === 'function' ? property() : property);
        propertyName = kebabCase(propertyName);
        statements.push(`  ${propertyName}: ${unwrappedProperty};`);
    });
    statements.push('}');
    return statements.join('\n');
}
function traverse(node, paths, styles, instance) {
    const pathIsString = typeof node.path === 'string';
    if (pathIsString)
        paths.push(node.path);
    else {
        if (node.path.before !== undefined)
            node.path.before(instance.context);
        paths.push(node.path.selector(instance.context));
    }
    const selector = parse_1.parseSelectorPath(paths, instance);
    const style = createStyle(selector, node.props, instance);
    if (style !== null)
        styles.push(style);
    if (node.children !== null) {
        node.children.forEach(childNode => {
            traverse(childNode, paths, styles, instance);
        });
    }
    paths.pop();
    if (!pathIsString && node.path.after !== undefined)
        node.path.after(instance.context);
}
function render(node, instance) {
    const styles = [];
    traverse(node, [], styles, instance);
    return styles.join('\n');
}
exports.render = render;
